=== modified file '__init__.py'
--- __init__.py	2005-08-30 19:22:37 +0000
+++ __init__.py	2005-08-31 18:41:38 +0000
@@ -30,11 +30,23 @@
         import clean_tree
         clean_tree.clean_tree(ignored=ignored)
 
+bzrlib.commands.OPTIONS['no-collapse'] = None
+
 class cmd_graph_ancestry(bzrlib.commands.Command):
+    """Produce ancestry graphs using dot.
+    
+    Output format is detected according to file extension.  Some of the more
+    common outputs are png, gif, svg, ps.  An extension of '.dot' will cause
+    a dot input file to be produced.
+
+    Ancestry is usually collapsed by removing nodes with a single parent
+    and descendant, but this can be disabled with --no-collapse.
+    """
     takes_args = ['branch', 'file']
-    def run(self, branch, file):
+    takes_options = ['no-collapse']
+    def run(self, branch, file, no_collapse=False):
         import graph
-        graph.write_ancestry_file(branch, file)
+        graph.write_ancestry_file(branch, file, not no_collapse)
 
 commands = [push.cmd_push, annotate.cmd_annotate, shelf.cmd_shelve, 
             shelf.cmd_unshelve, cmd_clean_tree, conflicts.cmd_conflicts,

=== modified file 'graph.py'
--- graph.py	2005-08-31 17:50:49 +0000
+++ graph.py	2005-08-31 18:41:38 +0000
@@ -24,9 +24,29 @@
     if len(new_committer) < 2:
         return committer
     return new_committer
-    
-
-def graph_ancestry(branch):
+
+
+def compact_descendants(descendants, ancestors):
+    new_descendants={}
+    skip = set()
+    for me, my_descendants in descendants.iteritems():
+        if me in skip:
+            continue
+        new_descendants[me] = []
+        for descendant in my_descendants:
+            new_descendant = descendant
+            while new_descendant in descendants and \
+                len(ancestors[new_descendant]) == 1 and \
+                len(descendants[new_descendant]) == 1:
+                skip.add(new_descendant)
+                if new_descendant in new_descendants:
+                    del new_descendants[new_descendant]
+                new_descendant = descendants[new_descendant][0]
+            new_descendants[me].append(new_descendant)
+    return new_descendants    
+
+
+def graph_ancestry(branch, collapse=True):
     nodes = {}
     q = ((i+1, n) for (i, n) in enumerate(branch.revision_history()))
     r = 1
@@ -60,33 +80,20 @@
                 descendants[parent].append(rev_id)
         lines = new_lines
     node_relations = []
-    def compact_descendants():
-        new_descendants={}
-        skip = set()
-        for me, my_descendants in descendants.iteritems():
-            if me in skip:
-                continue
-            new_descendants[me] = []
-            for descendant in my_descendants:
-                new_descendant = descendant
-                while new_descendant in descendants and \
-                    len(ancestors[new_descendant]) == 1 and \
-                    len(descendants[new_descendant]) == 1:
-                    skip.add(new_descendant)
-                    if new_descendant in new_descendants:
-                        del new_descendants[new_descendant]
-                    new_descendant = descendants[new_descendant][0]
-                new_descendants[me].append(new_descendant)
-        return new_descendants    
+
+    if collapse:
+        visible_descendants = compact_descendants(descendants, ancestors)
+    else:
+        visible_descendants = descendants
                 
-    for key, values in compact_descendants().iteritems():
+    for key, values in visible_descendants.iteritems():
         for value in values:
             node_relations.append((nodes[key], nodes[value]))
     return node_relations
 
-def write_ancestry_file(branch, filename):
+def write_ancestry_file(branch, filename, collapse=True):
     b = Branch(branch)
-    relations = graph_ancestry(b)
+    relations = graph_ancestry(b, collapse)
     ext = filename.split('.')[-1]
     if ext in ('svg', 'svgz', 'gif', 'jpg', 'ps', 'fig', 'mif', 'png'):
         invoke_dot(dot_output(relations), filename, ext)

